#!/usr/bin/env bash
# -
# - Syncing changes in the journal file for a given zone, or a zone related to
# - a given hostname
# -
# - Return (Exit) Codes:
# -    success:
# -       0:    Zone is synced, journalfile ist removed.
# -    error:
# -      10:    Invalid Hostname/Domain given
# -      11:    No zonefile found
# -      12:    Hostname/Domain not supported
# -      13:    Syncing zonefile with jounal file failed
# -      99:    Fatal error
# - 
# - Usage:   ./bind_rndc_sync_clean.sh <hostname|domain>
# -
# - Example: ./bind_rndc_sync_clean.sh a.mx.oopen.de


#---------------------------------------
#-----------------------------
# Setting Defaults
#-----------------------------
#---------------------------------------

DEFAULT_CONF_FILE_DIR="/etc/bind"
DEFAULT_BIND_USER="bind"
DEFAULT_BIND_GROUP="bind"


#***************************************
#----------------------------- 
# Don't make changes after this 
#----------------------------- 
#***************************************


working_dir="$(dirname $(realpath $0))"
conf_file="${working_dir}/conf/bind.conf"

log_file="$(mktemp)"


#---------------------------------------
#-----------------------------
# Base Function(s)
#-----------------------------
#---------------------------------------
usage() {
   echo 
      [ -n  "$1" ] && echo -e "Error: $1\n"

      cat<<EOF

   Usage: $(basename $0) <hostname|domain> | <check>

   Script syncs changes in the journal file for a given zone, or a zone related to
   a given hostname

   Return (Exit) Codes:
      success:
          0:    Zone is synced, journalfile ist removed.
      error:
         10:    Invalid Hostname/Domain given
         11:    No zonefile found
         12:    Hostname/Domain not supported
         13:    Syncing zonefile with jounal file failed
         99:    Fatal error
   

   Options:

      -h    
            Prints this help.

      -q
            Rund in silent mode.


   Example: $(basename $0) oopen.de

EOF
clean_up 99
}


clean_up() {

   # Perform program exit housekeeping
   rm $log_file
   exit $1
}

echononl(){
   if $verbose ; then
      echo X\\c > /tmp/shprompt$$
      if [ `wc -c /tmp/shprompt$$ | awk '{print $1}'` -eq 1 ]; then
         echo -e -n "$*\\c" 1>&2
      else
         echo -e -n "$*" 1>&2
      fi
      rm /tmp/shprompt$$
   fi
}

fatal(){
   echo ""
   echo -e "[ \033[31m\033[1mError\033[m ]: $*"
   echo ""
   echo -e "\t\033[31m\033[1mScript is canceled\033[m\033[m"
   echo ""
   clean_up 99
}

warn (){
   if $verbose ; then
      echo ""
      echo -e "\t[ \033[33m\033[1mWarning\033[m ]: $*"
      echo ""
   fi
}

info (){
   if $verbose ; then
      echo ""
      echo -e "\t[ \033[32m\033[1mInfo\033[m ]: $*"
      echo ""
   fi
}

ok (){
   if $verbose ; then
      echo ""
      echo -e "\t[ \033[36m\033[1mOk\033[m ]: $*"
      echo ""
   fi
}

error(){
   if $verbose ; then
      echo ""
      echo -e "\t[ \033[31m\033[1mFehler\033[m ]: $*"
      echo ""
   fi
}

echo_ok() {
   if $verbose ; then
      echo -e "\033[75G[ \033[32mok\033[m ]"
   fi
}
echo_failed(){
   if $verbose ; then
      echo -e "\033[75G[ \033[1;31mfailed\033[m ]"
   fi
}
echo_skipped() {
   if $verbose ; then
      echo -e "\033[75G[ \033[33m\033[1mskipped\033[m ]"
   fi
}

containsElement () {
   local e
   for e in "${@:2}"; do [[ "$e" == "$1" ]] && return 0; done
   return 1
}

trap clean_up SIGHUP SIGINT SIGTERM


# - Test whether stdout (file descriptor 1) is a terminal or not (e.g. cron
# - or if you pipe the output to some other program)
#
if [[ -t 1 ]] ; then
   verbose=true
else
   verbose=false
fi


while getopts hq opt ; do
   case $opt in
      q) verbose=false
         ;;
      h) usage
          ;;
      *)
         ;;
   esac
done

shift $(expr $OPTIND - 1)
if [[ $# -ne 1 ]] ; then
   if $verbose ; then
      usage "wrong number of arguments"
   else
      clean_up 99
   fi
fi

# - Parameter "check" can be used, to test whether this script
# - is accessable (e.g. from a script on a remote host)
# -
if [[ "$1" = "check" ]]; then
   info "Script \033[1m$(basename $0)\033[m was successfully invoked, but its only a test."
   clean_up 0
fi


host_name=$1


$verbose && echo ""

if [[ -z "$host_name" ]] ; then
   fatal "No hostname/domain given!"
fi



#---------------------------------------
#-----------------------------
# Load default values from bind.conf
#
# Overwrites the settings above
# 
#-----------------------------
#---------------------------------------

if $verbose ; then
   clear
   echo ""
   echo -e "\033[32mRunning script \033[1m"$(basename $0)"\033[m .."
   echo ""
fi


info "Given hostname/domain: \033[1m${host_name}\033[m"


echononl "\t   Loading default Configuration values from $(basename ${conf_file}).."
if [[ ! -f "$conf_file" ]]; then
   echo_skipped
else
   source "${conf_file}" > $log_file 2>&1
   if [[ $? -eq 0 ]]; then
      echo_ok
   else
      echo_failed
      fatal "$(cat $log_file)"
   fi
fi

[[ -n "$CONF_FILE_DIR" ]] || CONF_FILE_DIR="$DEFAULT_CONF_FILE_DIR"
[[ -n "$ZONES_DECLARATION_FILE" ]] || ZONES_DECLARATION_FILE="${CONF_FILE_DIR}/named.conf.local"
[[ -n "$BIND_USER" ]] || BIND_USER="$DEFAULT_BIND_USER"
[[ -n "$BIND_GROUP" ]] || BIND_GROUP="$DEFAULT_BIND_GROUP"

$verbose && echo ""



# - Validate Syntax of given domain
# -
valid_domain_regex="^([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9])(\.([a-zA-Z0-9]|[a-zA-Z0-9][a-zA-Z0-9\-]{0,61}[a-zA-Z0-9]))*$"
echononl "\tValidate syntax of given domain.."
if [[ $host_name =~ $valid_domain_regex ]]; then
   if [[ ! $host_name =~ \. ]]; then
       echo_failed
       error "Invalid hostname/domain \"$1\" given!"
       clean_up 10
   else
      echo_ok
   fi
else
   echo_failed
   error "Invalid hostname/domain ($1) given!"
   clean_up 10
fi


# - Determin zone (domain)
# -
_failed=false
_host_name=$host_name
_tmp_host_name=$(echo ${_host_name//\./\\.})
while ! grep -e "$_tmp_host_name" $ZONES_DECLARATION_FILE > /dev/null 2>&1 ; do
   _host_name=${_host_name#*.}
   _tmp_host_name=$(echo ${_host_name//\./\\.})
   if [[ ! $_tmp_host_name =~ \. ]]; then
      _failed=true
      break
   fi
done

if $_failed ; then
   error "Given hostname/domain \"$1\" not supported by this nameserver!"
   clean_up 11
else
   domain=$_host_name
fi


# - Determine zonefile (by reading bind configuration)
# -
_found=false
declare -i _number=0
regex_zone="^[[:space:]]*zone[[:space:]]+\"$_tmp_host_name\""
regex_file="^[[:space:]]*file"
while IFS='' read -r line || [[ -n "$line" ]] ; do
   if [[ $line  =~ $regex_zone ]]; then
      _found=true
   fi
   if $_found ; then
      if [[ $line  =~ $regex_file ]]; then
         zone_file=`echo $line | awk '{print$2}'`
         shopt -s extglob
         if [[ $zone_file =~ \; ]]; then
            zone_file=${zone_file%%*(\;)}
         fi
         if [[ $zone_file =~ ^\" ]]; then
            zone_file=${zone_file##*(\")}
            zone_file=${zone_file%%*(\")}
         fi
         shopt -u extglob
         let number++
         break
      fi
   fi
done < $ZONES_DECLARATION_FILE

if [[ $number -eq 0 ]] ; then
   error "No Zonefile (master) found for domain \"$domain\" ."
   clean_up 12
fi


# - Reload Zone
# -
echononl "\tSyncing changes in the journal file for zone \"$domain\".."
#rndc sync -clean  $domain > /dev/null 2>&1
rndc sync $domain > /dev/null 2>&1
if [[ $? -eq 0 ]]; then
   echo_ok
   info "Zone was synced ($domain)"
   clean_up 0
else
   echo_failed
   error "Syncing Zone ($domain) failed!"
   clean_up 13
fi

$verbose && echo ""
clean_up 99
